@@ -15,6 +15,7 @@
 /*			    if (func(arg) < 0)				*/
 /*			becomes,					*/
 /*			    if (func(arg) > 10)				*/
+/* 08May2001:	ACR	Started using Dan's strerr_warn etc. calls.	*/
 /************************************************************************/
 /* To do:	Standardise return codes.				*/
 /************************************************************************/
@@ -24,18 +25,21 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include "fmt.h"
+#include "str.h"
+#include "strerr.h"
+#include "stralloc.h"
+#include "substdio.h"
+#include "subfd.h"
+#include "error.h"
+
 #include "hash_core.h"
 #include "admin_core.h"
 #include "noddylib.h"
-#include "../qmail-1.03/fmt.h"
-#include "../qmail-1.03/str.h"
-#include "../qmail-1.03/substdio.h"
-#include "../qmail-1.03/subfd.h"
-#include "../qmail-1.03/error.h"
 
 /* chdir_or_mkdir... tries to change to a directory, or failing that,	*/
 /* creates it and then changes into it. In the former case it returns	*/
-/* 0, in the latter 1. Otherwise a return of -1 indicates an error.	*/
+/* 0, in the latter 1. Otherwise a return of 11 indicates an error.	*/
 
 int chdir_or_mkdir700_and_chdir(char *dir)
 {
@@ -46,11 +50,9 @@ int chdir_or_mkdir700_and_chdir(char *dir)
       um=umask(0077);
       if ((mkdir(dir,0700) < 0) || (chdir(dir) < 0))
       {
-        substdio_puts(subfderr,"Error when trying to mkdir/chdir: ");
-        substdio_puts(subfderr,error_str(errno));
-        substdio_puts(subfderr,"\n");
-        substdio_flush(subfderr);
-        umask(um); return(11);
+        strerr_warn1("Error when trying to mkdir/chdir: ",&strerr_sys);
+        umask(um);
+        return(11);
       };
 
       umask(um);
@@ -58,10 +60,7 @@ int chdir_or_mkdir700_and_chdir(char *dir)
     }
     else			/* Non-ENOENT problem with chdir	*/
     {
-      substdio_puts(subfderr,"Unexpected error: ");
-      substdio_puts(subfderr,error_str(errno));
-      substdio_puts(subfderr,"\n");
-      substdio_flush(subfderr);
+      strerr_warn1("Unexpected error: ",&strerr_sys);
       return(11);
     };
   return(0);					/* Dir already exists	*/
@@ -89,45 +88,30 @@ int add_user(char *u)
 
   if (stat(maildirs_basedir,&st) < 0)
   {
-    substdio_puts(subfderr,"Error: ");
-    substdio_puts(subfderr,maildirs_basedir);
-    substdio_puts(subfderr,": ");
-    substdio_puts(subfderr,error_str(errno));
-    substdio_puts(subfderr,"\n");
+    strerr_warn3("Error: ",maildirs_basedir,": ", &strerr_sys);
     substdio_flush(subfderr);
     return(11);
   };
   if (st.st_uid != geteuid())
   {
-    substdio_puts(subfderr,"Error: Ownership of ");
-    substdio_puts(subfderr,maildirs_basedir);
-    substdio_puts(subfderr," is wrong, or you're running this");
-    substdio_puts(subfderr," program as the wrong user.\n");
-    substdio_puts(subfderr,"\n");
-    substdio_flush(subfderr);
+    strerr_warn4("Error: Ownership of ",maildirs_basedir," is wrong, or",
+                 " you're running this program as the wrong user.",0);
     return(11);
   };
   if (chdir(maildirs_basedir) < 0)
   {
-    substdio_puts(subfderr,"Error entering ");
-    substdio_puts(subfderr,maildirs_basedir);
-    substdio_puts(subfderr,": ");
-    substdio_puts(subfderr,error_str(errno));
-    substdio_puts(subfderr,"\n");
-    substdio_flush(subfderr);
+    strerr_warn3("Error entering ",maildirs_basedir,": ",&strerr_sys);
     return(12);
   };
   if (add_hashdirs_and_chdir(u) > 10) return(11);
   if ( res=chdir_or_mkdir700_and_chdir(u), res > 10  )
   {
-    substdio_puts(subfderr,"Couldn't make actual user dir\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Couldn't make actual user dir",0);
     return(13);
   };
   if ( res==0 )
   {
-    substdio_puts(subfderr,"Error: User dir already exists.\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: User dir already exists.",0);
     return(14);
   }
   else
@@ -135,7 +119,8 @@ int add_user(char *u)
 }
 
 int add_to_aliaslist(char *a)
-{
+{ /* Thinks: May prefer to have hashdir_a as a parameter too, since this
+     is known when this function is called */
   char hashdir_a[MAX_HASHDIR_LEN];
   char *p;
   int res;
@@ -144,10 +129,7 @@ int add_to_aliaslist(char *a)
   get_hashdir(a,hashdir_a);
   if (chdir(hashdir_a) < 0)
   {
-    substdio_puts(subfderr,"Error: Couldn't enter directory for ");
-    substdio_puts(subfderr,a);
-    substdio_puts(subfderr,"\n");
-    substdio_flush(subfderr);
+    strerr_warn2("Error: Couldn't enter directory for ",a,0);
     return(12);
   };
   if (chdir_or_mkdir700_and_chdir(ALIAS_LIST) > 10)
@@ -155,8 +137,7 @@ int add_to_aliaslist(char *a)
   /* Don't need fd returned by open: File must merely exist */
   if (open(a,O_WRONLY | O_EXCL | O_CREAT,0400) < 0)
   {
-    substdio_puts(subfderr,"Error: Unexpected problem updating the aliases list.\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: Unexpected problem updating the aliases list.",0);
     return(12);
   };
   return(0);	/* Success: symlink & alias list entry added	*/
@@ -165,77 +146,56 @@ int add_to_aliaslist(char *a)
 
 int add_alias(char *u, char *a)
 {
-  char hashdir_u[MAX_HASHDIR_LEN], hashdir_a[MAX_HASHDIR_LEN],
-       syml[MAX_HASH_LEN+9+MAX_HASHABLE_LEN]; /* ../../../h1/h2/h3/username */
-  char *p;
+  stralloc hashdir_u_sa = {0}, hashdir_a_sa = {0}, syml_sa = {0};
   int res;
   struct stat st,lst;
 
   if (stat(maildirs_basedir,&st) < 0)
   {
-    substdio_puts(subfderr,"Error: ");
-    substdio_puts(subfderr,maildirs_basedir);
-    substdio_puts(subfderr,": ");
-    substdio_puts(subfderr,error_str(errno));
-    substdio_puts(subfderr,"\n");
-    substdio_flush(subfderr);
+    strerr_warn3("Error: ",maildirs_basedir,": ",&strerr_sys);
     return(11);
   };
   if (st.st_uid != geteuid())
   {
-    substdio_puts(subfderr,"Error: Ownership of ");
-    substdio_puts(subfderr,maildirs_basedir);
-    substdio_puts(subfderr," is wrong, or you're running this");
-    substdio_puts(subfderr," program as the wrong user.\n");
-    substdio_flush(subfderr);
+    strerr_warn4("Error: Ownership of ",maildirs_basedir," is wrong,",
+                 " or you're running this program as the wrong user.",0);
     return(11);
   };
-  get_hashdir(u,hashdir_u);
-  get_hashdir(a,hashdir_a);
 
-  if (chdir(hashdir_u) < 0)
+  add_hashdir_sa(u,&hashdir_u_sa);
+  add_hashdir_sa(a,&hashdir_a_sa);
+
+  if (chdir(hashdir_u_sa.s) < 0)
   {
-    substdio_puts(subfderr,"Error: User not setup.\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: User not setup.",0);
     return(12);
   };
-  if (chdir(hashdir_a) == 0)
+  if (chdir(hashdir_a_sa.s) == 0)
   {
-    substdio_puts(subfderr,"Error: Alias/username already in use.\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: Alias/username already in use.",0);
     return(12);
   };
-  if (lstat(hashdir_u,&lst) < 0)
+  if (lstat(hashdir_u_sa.s,&lst) < 0)
   {
-    substdio_puts(subfderr,"Error: Unexpected problem with username.\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: Unexpected problem with username.",0);
     return(12);
   };
   if (S_ISLNK(lst.st_mode))
   {
-    substdio_puts(subfderr,"Error: Username given is actually an alias.\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: Username given is actually an alias.",0);
     return(12);
   };
   if (chdir(maildirs_basedir) < 0)
   {
-    substdio_puts(subfderr,"Error entering ");
-    substdio_puts(subfderr,maildirs_basedir);
-    substdio_puts(subfderr,": ");
-    substdio_puts(subfderr,error_str(errno));
-    substdio_puts(subfderr,"\n");
-    substdio_flush(subfderr);
+    strerr_warn3("Error entering ",maildirs_basedir,": ",&strerr_sys);
     return(12);
   };
   if (add_hashdirs_and_chdir(a) > 10) return(11);
-  p=syml+fmt_str(syml,"../../../"); *p='\0';
-  get_hashes_str(u,p); /* adds hashes after "/" */
-  if ( symlink(syml,hashdir_a) < 0 )
+  if (!stralloc_cats(&syml_sa,"../../../")) die_nomem111();
+  add_hash_sa(u,&syml_sa);	/* ../../../h1/h2/h3/username */
+  if ( symlink(syml_sa.s,hashdir_a_sa.s) < 0 )
   {
-    substdio_puts(subfderr,"Error: Couldn't make alias: ");
-    substdio_puts(subfderr,error_str(errno));
-    substdio_puts(subfderr,"\n");
-    substdio_flush(subfderr);
+    strerr_warn1("Error: Couldn't make alias: ",&strerr_sys);
     return(13);
   };
   return(add_to_aliaslist(a));
