@@ -7,12 +7,12 @@
 /* Authors:	Andrew Richards andrew@tic.ch (Advert: I offer qmail	*/
 /*		  consultancy services - ich spreche auch Deutsch)	*/
 /************************************************************************/
+/* 09May2001	ACR	Removed function prototypes for non-existent	*/
+/*			functions!					*/
+/************************************************************************/
 
 void tidystringtail(char *str);		/* Removes trailing white space */
-void capitalise(char *str);
-void decapitalise(char *str);
-void convertpopstring(char *popstring);	/* % becomes @ */
-char *stripusername(char *s);
-void stringclean(char *popstring);	/* tidy+decap+xvert */
+void convertpopstring(char *popstring);	/* % becomes @			*/
+char *stripusername(char *s);		/* "@domain" from "user@domain"	*/
+void stringclean(char *popstring);	/* tidy+decap+xvert		*/
 int str_checkchars(char *s,char *validchars);
-struct passwd *getpwnam(const char *name);
